@@ -43,7 +43,8 @@ function correlation_3op_2t(
         ψ0 = steadystate(L)
     end
 
-    check_dimensions(L, ψ0, A, B, C)
+    check_mul_dimensions(L, ψ0)
+    check_dimensions(L, A, B, C)
 
     kwargs2 = merge((saveat = collect(tlist),), (; kwargs...))
     ρt_list = mesolve(L, ψ0, tlist; kwargs2...).states
@@ -113,7 +114,7 @@ function correlation_2op_2t(
         reverse::Bool = false,
         kwargs...,
     ) where {HOpType <: Union{Operator, SuperOperator}, StateOpType <: Union{Ket, Operator}}
-    C = eye(prod(H.dimensions), dims = H.dimensions)
+    C = eye(get_hilbert_size(H.dimensions)[1], dims = H.dimensions)
     if reverse
         corr = correlation_3op_2t(H, ψ0, tlist, τlist, c_ops, A, B, C; kwargs...)
     else
