@@ -50,78 +50,52 @@ for op in (:(+), :(-), :(*))
     end
 end
 
-function check_mul_dimensions(from::NTuple{NA, AbstractSpace}, to::NTuple{NB, AbstractSpace}) where {NA, NB}
-    (from != to) && throw(
+function check_mul_dimensions(A::AbstractQuantumObject, B::AbstractQuantumObject)
+    (A.dimensions.from != B.dimensions.to) && throw(
         DimensionMismatch(
-            "The quantum object with (right) dims = $(dimensions_to_dims(from)) can not multiply a quantum object with (left) dims = $(dimensions_to_dims(to)) on the right-hand side.",
+            "The quantum object with dims = $(A.dims) can not multiply a quantum object with dims = $(B.dims) on the right-hand side.",
         ),
     )
     return nothing
 end
 
-for ADimType in (:Dimensions, :GeneralDimensions)
-    for BDimType in (:Dimensions, :GeneralDimensions)
-        if ADimType == BDimType == :Dimensions
-            @eval begin
-                function Base.:(*)(
-                        A::AbstractQuantumObject{Operator, <:$ADimType},
-                        B::AbstractQuantumObject{Operator, <:$BDimType},
-                    )
-                    check_dimensions(A, B)
-                    QType = promote_op_type(A, B)
-                    return QType(A.data * B.data, Operator(), A.dimensions)
-                end
-            end
-        else
-            @eval begin
-                function Base.:(*)(
-                        A::AbstractQuantumObject{Operator, <:$ADimType},
-                        B::AbstractQuantumObject{Operator, <:$BDimType},
-                    )
-                    check_mul_dimensions(get_dimensions_from(A), get_dimensions_to(B))
-                    QType = promote_op_type(A, B)
-                    return QType(
-                        A.data * B.data,
-                        Operator(),
-                        GeneralDimensions(get_dimensions_to(A), get_dimensions_from(B)),
-                    )
-                end
-            end
-        end
-    end
+function Base.:(*)(A::AbstractQuantumObject{Operator}, B::AbstractQuantumObject{Operator})
+    check_mul_dimensions(A, B)
+    QType = promote_op_type(A, B)
+    return QType(A.data * B.data, Operator(), ProductDimensions(A.dimensions.to, B.dimensions.from))
 end
 
-function Base.:(*)(A::AbstractQuantumObject{Operator}, B::QuantumObject{Ket, <:Dimensions})
-    check_mul_dimensions(get_dimensions_from(A), get_dimensions_to(B))
-    return QuantumObject(A.data * B.data, Ket(), Dimensions(get_dimensions_to(A)))
+function Base.:(*)(A::AbstractQuantumObject{Operator}, B::QuantumObject{Ket})
+    check_mul_dimensions(A, B)
+    return QuantumObject(A.data * B.data, Ket(), ProductDimensions(A.dimensions.to, B.dimensions.from))
 end
-function Base.:(*)(A::QuantumObject{Bra, <:Dimensions}, B::AbstractQuantumObject{Operator})
-    check_mul_dimensions(get_dimensions_from(A), get_dimensions_to(B))
-    return QuantumObject(A.data * B.data, Bra(), Dimensions(get_dimensions_from(B)))
+function Base.:(*)(A::QuantumObject{Bra}, B::AbstractQuantumObject{Operator})
+    check_mul_dimensions(A, B)
+    return QuantumObject(A.data * B.data, Bra(), ProductDimensions(A.dimensions.to, B.dimensions.from))
 end
 function Base.:(*)(A::QuantumObject{Ket}, B::QuantumObject{Bra})
-    check_dimensions(A, B)
-    return QuantumObject(A.data * B.data, Operator(), A.dimensions) # to align with QuTiP, don't use kron(A, B) to do it.
+    check_mul_dimensions(A, B)
+    return QuantumObject(A.data * B.data, Operator(), ProductDimensions(A.dimensions.to, B.dimensions.from))
 end
 function Base.:(*)(A::QuantumObject{Bra}, B::QuantumObject{Ket})
-    check_dimensions(A, B)
+    check_mul_dimensions(A, B)
     return A.data * B.data
 end
 function Base.:(*)(A::AbstractQuantumObject{SuperOperator}, B::QuantumObject{Operator})
-    check_dimensions(A, B)
-    return QuantumObject(vec2mat(A.data * mat2vec(B.data)), Operator(), A.dimensions)
+    check_mul_dimensions(A, B)
+    return QuantumObject(vec2mat(A.data * mat2vec(B.data)), Operator(), A.dimensions.to)
 end
 function Base.:(*)(A::QuantumObject{OperatorBra}, B::QuantumObject{OperatorKet})
-    check_dimensions(A, B)
+    check_mul_dimensions(A, B)
     return A.data * B.data
 end
 function Base.:(*)(A::AbstractQuantumObject{SuperOperator}, B::QuantumObject{OperatorKet})
-    check_dimensions(A, B)
-    return QuantumObject(A.data * B.data, OperatorKet(), A.dimensions)
+    check_mul_dimensions(A, B)
+    return QuantumObject(A.data * B.data, OperatorKet(), A.dimensions.to)
 end
 function Base.:(*)(A::QuantumObject{OperatorBra}, B::AbstractQuantumObject{SuperOperator})
-    check_dimensions(A, B)
-    return QuantumObject(A.data * B.data, OperatorBra(), A.dimensions)
+    check_mul_dimensions(A, B)
+    return QuantumObject(A.data * B.data, OperatorBra(), B.dimensions.from)
 end
 
 Base.:(^)(A::QuantumObject, n::T) where {T <: Number} = QuantumObject(^(A.data, n), A.type, A.dimensions)
@@ -157,7 +131,8 @@ Supports the following inputs:
     `matrix_element(i, A, j)` is a synonym of `dot(i, A, j)`.
 """
 function LinearAlgebra.dot(i::QuantumObject{Ket}, A::AbstractQuantumObject{Operator}, j::QuantumObject{Ket})
-    check_dimensions(i, A, j)
+    check_mul_dimensions(i', A)
+    check_mul_dimensions(A, j)
     return LinearAlgebra.dot(i.data, A.data, j.data)
 end
 function LinearAlgebra.dot(
@@ -165,7 +140,8 @@ function LinearAlgebra.dot(
         A::AbstractQuantumObject{SuperOperator},
         j::QuantumObject{OperatorKet},
     )
-    check_dimensions(i, A, j)
+    check_mul_dimensions(i', A)
+    check_mul_dimensions(A, j)
     return LinearAlgebra.dot(i.data, A.data, j.data)
 end
 
@@ -241,7 +217,7 @@ Note that this function only supports for [`Operator`](@ref) and [`SuperOperator
 ```jldoctest
 julia> a = destroy(20)
 
-Quantum Object:   type=Operator()   dims=[20]   size=(20, 20)   ishermitian=false
+Quantum Object:   type=Operator()   dims=([20], [20])   size=(20, 20)   ishermitian=false
 20×20 SparseMatrixCSC{ComplexF64, Int64} with 19 stored entries:
 ⎡⠈⠢⡀⠀⠀⠀⠀⠀⠀⠀⎤
 ⎢⠀⠀⠈⠢⡀⠀⠀⠀⠀⠀⎥
@@ -278,7 +254,7 @@ Return the standard vector `p`-norm or [Schatten](https://en.wikipedia.org/wiki/
 ```jldoctest
 julia> ψ = fock(10, 2)
 
-Quantum Object:   type=Ket()   dims=[10]   size=(10,)
+Quantum Object:   type=Ket()   dims=([10], [1])   size=(10,)
 10-element Vector{ComplexF64}:
  0.0 + 0.0im
  0.0 + 0.0im
@@ -360,7 +336,7 @@ Matrix square root of [`QuantumObject`](@ref)
 !!! note
     `√(A)` (where `√` can be typed by tab-completing `\sqrt` in the REPL) is a synonym of `sqrt(A)`.
 """
-Base.sqrt(A::QuantumObject) = QuantumObject(sqrt(to_dense(A.data)), A.type, A.dimensions)
+Base.sqrt(A::QuantumObject{ObjType}) where {ObjType <: Union{Operator, SuperOperator}} = QuantumObject(sqrt(to_dense(A.data)), A.type, A.dimensions)
 
 @doc raw"""
     log(A::QuantumObject)
@@ -470,7 +446,7 @@ Two qubits in the state ``\ket{\psi} = \ket{e,g}``:
 ```jldoctest
 julia> ψ = kron(fock(2,0), fock(2,1))
 
-Quantum Object:   type=Ket()   dims=[2, 2]   size=(4,)
+Quantum Object:   type=Ket()   dims=([2, 2], [1, 1])   size=(4,)
 4-element Vector{ComplexF64}:
  0.0 + 0.0im
  1.0 + 0.0im
@@ -479,7 +455,7 @@ Quantum Object:   type=Ket()   dims=[2, 2]   size=(4,)
 
 julia> ptrace(ψ, 2)
 
-Quantum Object:   type=Operator()   dims=[2]   size=(2, 2)   ishermitian=true
+Quantum Object:   type=Operator()   dims=([2], [2])   size=(2, 2)   ishermitian=true
 2×2 Matrix{ComplexF64}:
  0.0+0.0im  0.0+0.0im
  0.0+0.0im  1.0+0.0im
@@ -489,7 +465,7 @@ or in an entangled state ``\ket{\psi} = \frac{1}{\sqrt{2}} \left( \ket{e,e} + \k
 ```jldoctest
 julia> ψ = 1 / √2 * (kron(fock(2,0), fock(2,0)) + kron(fock(2,1), fock(2,1)))
 
-Quantum Object:   type=Ket()   dims=[2, 2]   size=(4,)
+Quantum Object:   type=Ket()   dims=([2, 2], [1, 1])   size=(4,)
 4-element Vector{ComplexF64}:
  0.7071067811865475 + 0.0im
                 0.0 + 0.0im
@@ -498,7 +474,7 @@ Quantum Object:   type=Ket()   dims=[2, 2]   size=(4,)
 
 julia> ptrace(ψ, 1)
 
-Quantum Object:   type=Operator()   dims=[2]   size=(2, 2)   ishermitian=true
+Quantum Object:   type=Operator()   dims=([2], [2])   size=(2, 2)   ishermitian=true
 2×2 Matrix{ComplexF64}:
  0.5+0.0im  0.0+0.0im
  0.0+0.0im  0.5+0.0im
@@ -522,8 +498,9 @@ function ptrace(QO::QuantumObject{Ket}, sel::Union{AbstractVector{Int}, Tuple})
     end
 
     _sort_sel = sort(SVector{length(sel), Int}(sel))
-    ρtr, dkeep = _ptrace_ket(QO.data, QO.dims, _sort_sel)
-    return QuantumObject(ρtr, type = Operator(), dims = Dimensions(dkeep))
+    dims = dimensions_to_dims(QO.dimensions.to)
+    ρtr, dkeep = _ptrace_ket(QO.data, dims, _sort_sel)
+    return QuantumObject(ρtr, type = Operator(), dims = ProductDimensions(dkeep))
 end
 
 ptrace(QO::QuantumObject{Bra}, sel::Union{AbstractVector{Int}, Tuple}) = ptrace(QO', sel)
@@ -532,8 +509,7 @@ function ptrace(QO::QuantumObject{Operator}, sel::Union{AbstractVector{Int}, Tup
     any(s -> s isa EnrSpace, QO.dimensions.to) && throw(ArgumentError("ptrace does not support EnrSpace"))
 
     # TODO: support for special cases when some of the subsystems have same `to` and `from` space
-    isa(QO.dimensions, GeneralDimensions) &&
-        (get_dimensions_to(QO) != get_dimensions_from(QO)) &&
+    !isendomorphism(QO.dimensions) &&
         throw(ArgumentError("Invalid partial trace for dims = $(_get_dims_string(QO.dimensions))"))
 
     _non_static_array_warning("sel", sel)
@@ -550,10 +526,10 @@ function ptrace(QO::QuantumObject{Operator}, sel::Union{AbstractVector{Int}, Tup
         (n_d == 1) && return QO
     end
 
-    dims = dimensions_to_dims(get_dimensions_to(QO))
+    dims = dimensions_to_dims(QO.dimensions.to)
     _sort_sel = sort(SVector{length(sel), Int}(sel))
     ρtr, dkeep = _ptrace_oper(QO.data, dims, _sort_sel)
-    return QuantumObject(ρtr, type = Operator(), dims = Dimensions(dkeep))
+    return QuantumObject(ρtr, type = Operator(), dims = ProductDimensions(dkeep))
 end
 ptrace(QO::QuantumObject, sel::Int) = ptrace(QO, SVector(sel))
 
@@ -672,7 +648,7 @@ Get the coherence value ``\alpha`` by measuring the expectation value of the des
 It returns both ``\alpha`` and the corresponding state with the coherence removed: ``\ket{\delta_\alpha} = \exp ( \alpha^* \hat{a} - \alpha \hat{a}^\dagger ) \ket{\psi}`` for a pure state, and ``\hat{\rho_\alpha} = \exp ( \alpha^* \hat{a} - \alpha \hat{a}^\dagger ) \hat{\rho} \exp ( -\bar{\alpha} \hat{a} + \alpha \hat{a}^\dagger )`` for a density matrix. These states correspond to the quantum fluctuations around the coherent state ``\ket{\alpha}`` or ``|\alpha\rangle\langle\alpha|``.
 """
 function get_coherence(ψ::QuantumObject{Ket})
-    a = destroy(prod(ψ.dimensions))
+    a = destroy(get_hilbert_size(ψ.dimensions)[1])
     α = expect(a, ψ)
     D = exp(α * a' - conj(α) * a)
 
@@ -680,7 +656,7 @@ function get_coherence(ψ::QuantumObject{Ket})
 end
 
 function get_coherence(ρ::QuantumObject{Operator})
-    a = destroy(prod(ρ.dimensions))
+    a = destroy(get_hilbert_size(ρ.dimensions)[1])
     α = expect(a, ρ)
     D = exp(α * a' - conj(α) * a)
 
@@ -730,24 +706,25 @@ function SparseArrays.permute(
     order_svector = SVector{length(order), Int}(order) # convert it to SVector for performance
 
     # obtain the arguments: dims for reshape; perm for PermutedDimsArray
-    dims, perm = _dims_and_perm(A.type, A.dims, order_svector, length(order_svector))
+    dims, perm = _dims_and_perm(A.type, A.dimensions, order_svector, length(order_svector))
 
-    order_dimensions = _order_dimensions(A.dimensions, order_svector)
+    order_dimensions = _order_dimensions(A.type, A.dimensions, order_svector)
 
     return QuantumObject(reshape(permutedims(reshape(A.data, dims...), Tuple(perm)), size(A)), A.type, order_dimensions)
 end
 
-_dims_and_perm(::ObjType, dims::SVector{N, Int}, order::AbstractVector{Int}, L::Int) where {ObjType <: Union{Ket, Bra}, N} =
-    reverse(dims), reverse((L + 1) .- order)
+_dims_and_perm(::Ket, dimensions::ProductDimensions, order::AbstractVector{Int}, L::Int) =
+    reverse(dimensions_to_dims(dimensions.to)), reverse((L + 1) .- order)
 
-# if dims originates from Dimensions
-_dims_and_perm(::Operator, dims::SVector{N, Int}, order::AbstractVector{Int}, L::Int) where {N} =
-    reverse(vcat(dims, dims)), reverse((2 * L + 1) .- vcat(order, order .+ L))
+_dims_and_perm(::Bra, dimensions::ProductDimensions, order::AbstractVector{Int}, L::Int) =
+    reverse(dimensions_to_dims(dimensions.from)), reverse((L + 1) .- order)
 
-# if dims originates from GeneralDimensions
-_dims_and_perm(::Operator, dims::SVector{2, SVector{N, Int}}, order::AbstractVector{Int}, L::Int) where {N} =
-    reverse(vcat(dims[2], dims[1])), reverse((2 * L + 1) .- vcat(order, order .+ L))
+_dims_and_perm(::Operator, dimensions::ProductDimensions, order::AbstractVector{Int}, L::Int) =
+    reverse(vcat(dimensions_to_dims(dimensions.from), dimensions_to_dims(dimensions.to))), reverse((2 * L + 1) .- vcat(order, order .+ L))
 
-_order_dimensions(dimensions::Dimensions, order::AbstractVector{Int}) = Dimensions(dimensions.to[order])
-_order_dimensions(dimensions::GeneralDimensions, order::AbstractVector{Int}) =
-    GeneralDimensions(dimensions.to[order], dimensions.from[order])
+_order_dimensions(::Ket, dimensions::ProductDimensions, order::AbstractVector{Int}) =
+    ProductDimensions(dimensions.to[order], dimensions.from)  # from is one-list, keep as is
+_order_dimensions(::Bra, dimensions::ProductDimensions, order::AbstractVector{Int}) =
+    ProductDimensions(dimensions.to, dimensions.from[order])  # to is one-list, keep as is
+_order_dimensions(::Operator, dimensions::ProductDimensions, order::AbstractVector{Int}) =
+    ProductDimensions(dimensions.to[order], dimensions.from[order])
