@@ -137,24 +137,27 @@ end
 to_dense(::Type{M}) where {M <: AbstractMatrix} = M
 
 @doc raw"""
-    to_sparse(A::QuantumObject)
+    to_sparse(A::QuantumObject, tol::Real = settings.tidyup_tol)
 
-Converts a dense QuantumObject to a sparse QuantumObject.
+Converts a dense QuantumObject to a sparse QuantumObject by removing elements with absolute values smaller than `tol`.
 """
-to_sparse(A::QuantumObject, tol::Real = 1.0e-10) = QuantumObject(to_sparse(A.data, tol), A.type, A.dimensions)
-function to_sparse(A::MT, tol::Real = 1.0e-10) where {MT <: AbstractMatrix}
+to_sparse(A::QuantumObject, tol::Real = settings.tidyup_tol) = QuantumObject(to_sparse(A.data, tol), A.type, A.dimensions)
+function to_sparse(A::MT, tol::Real = settings.tidyup_tol) where {MT <: AbstractMatrix}
     idxs = findall(@. abs(A) > tol)
     row_indices = getindex.(idxs, 1)
     col_indices = getindex.(idxs, 2)
     vals = getindex(A, idxs)
     return sparse(row_indices, col_indices, vals, size(A)...)
 end
-function to_sparse(A::VT, tol::Real = 1.0e-10) where {VT <: AbstractVector}
+function to_sparse(A::VT, tol::Real = settings.tidyup_tol) where {VT <: AbstractVector}
     idxs = findall(@. abs(A) > tol)
     vals = getindex(A, idxs)
     return sparsevec(idxs, vals, length(A))
 end
 
+to_sparse_if_needed(::Val{needed}, A::Union{QuantumObject, AbstractArray}, tol::Real = settings.tidyup_tol) where {needed} =
+    needed ? to_sparse(A, tol) : A
+
 @doc raw"""
     kron(A::AbstractQuantumObject, B::AbstractQuantumObject, ...)
     tensor(A::AbstractQuantumObject, B::AbstractQuantumObject, ...)
@@ -171,7 +174,7 @@ Returns the [Kronecker product](https://en.wikipedia.org/wiki/Kronecker_product)
 ```jldoctest
 julia> a = destroy(20)
 
-Quantum Object:   type=Operator()   dims=[20]   size=(20, 20)   ishermitian=false
+Quantum Object:   type=Operator()   dims=([20], [20])   size=(20, 20)   ishermitian=false
 20×20 SparseMatrixCSC{ComplexF64, Int64} with 19 stored entries:
 ⎡⠈⠢⡀⠀⠀⠀⠀⠀⠀⠀⎤
 ⎢⠀⠀⠈⠢⡀⠀⠀⠀⠀⠀⎥
@@ -185,44 +188,26 @@ julia> size(a), size(O)
 ((20, 20), (400, 400))
 
 julia> a.dims, O.dims
-([20], [20, 20])
+(([20], [20]), ([20, 20], [20, 20]))
 ```
 """
 function Base.kron(
-        A::AbstractQuantumObject{OpType, <:Dimensions},
-        B::AbstractQuantumObject{OpType, <:Dimensions},
+        A::AbstractQuantumObject{OpType},
+        B::AbstractQuantumObject{OpType},
     ) where {OpType <: Union{Ket, Bra, Operator}}
     QType = promote_op_type(A, B)
     _lazy_tensor_warning(A.data, B.data)
-    return QType(kron(A.data, B.data), A.type, Dimensions((A.dimensions.to..., B.dimensions.to...)))
+    return QType(
+        kron(A.data, B.data),
+        A.type,
+        ProductDimensions(
+            (A.dimensions.to..., B.dimensions.to...),
+            (A.dimensions.from..., B.dimensions.from...),
+        ),
+    )
 end
 
-# if A and B are both Operator but either one of them has GeneralDimensions
-for ADimType in (:Dimensions, :GeneralDimensions)
-    for BDimType in (:Dimensions, :GeneralDimensions)
-        if !(ADimType == BDimType == :Dimensions) # not for this case because it's already implemented
-            @eval begin
-                function Base.kron(
-                        A::AbstractQuantumObject{Operator, <:$ADimType},
-                        B::AbstractQuantumObject{Operator, <:$BDimType},
-                    )
-                    QType = promote_op_type(A, B)
-                    _lazy_tensor_warning(A.data, B.data)
-                    return QType(
-                        kron(A.data, B.data),
-                        Operator(),
-                        GeneralDimensions(
-                            (get_dimensions_to(A)..., get_dimensions_to(B)...),
-                            (get_dimensions_from(A)..., get_dimensions_from(B)...),
-                        ),
-                    )
-                end
-            end
-        end
-    end
-end
-
-# if A and B are different type (must return Operator with GeneralDimensions)
+# if A and B are different type (must return Operator)
 for AOpType in (:Ket, :Bra, :Operator)
     for BOpType in (:Ket, :Bra, :Operator)
         if (AOpType != BOpType)
@@ -233,9 +218,9 @@ for AOpType in (:Ket, :Bra, :Operator)
                     return QType(
                         kron(A.data, B.data),
                         Operator(),
-                        GeneralDimensions(
-                            (get_dimensions_to(A)..., get_dimensions_to(B)...),
-                            (get_dimensions_from(A)..., get_dimensions_from(B)...),
+                        ProductDimensions(
+                            (A.dimensions.to..., B.dimensions.to...),
+                            (A.dimensions.from..., B.dimensions.from...),
                         ),
                     )
                 end
@@ -269,7 +254,7 @@ Convert a quantum object from vector ([`OperatorKet`](@ref)-type) to matrix ([`O
 !!! note
     `vector_to_operator` is a synonym of `vec2mat`.
 """
-vec2mat(A::QuantumObject{OperatorKet}) = QuantumObject(vec2mat(A.data), Operator(), A.dimensions)
+vec2mat(A::QuantumObject{OperatorKet}) = QuantumObject(vec2mat(A.data), Operator(), A.dimensions.to)
 
 @doc raw"""
     mat2vec(A::QuantumObject)
@@ -280,7 +265,10 @@ Convert a quantum object from matrix ([`Operator`](@ref)-type) to vector ([`Oper
 !!! note
     `operator_to_vector` is a synonym of `mat2vec`.
 """
-mat2vec(A::QuantumObject{Operator}) = QuantumObject(mat2vec(A.data), OperatorKet(), A.dimensions)
+function mat2vec(A::QuantumObject{Operator})
+    isendomorphism(A.dimensions) || throw(ArgumentError("mat2vec requires a square Operator (same to and from dimensions)."))
+    return QuantumObject(mat2vec(A.data), OperatorKet(), A.dimensions.to)
+end
 
 @doc raw"""
     mat2vec(A::AbstractMatrix)
