@@ -29,7 +29,7 @@ This operator can be initialized in the same way as the QuTiP `QobjEvo` object.
 ```jldoctest qobjevo
 julia> a = tensor(destroy(10), qeye(2))
 
-Quantum Object:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=false
+Quantum Object:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=false
 20×20 SparseMatrixCSC{ComplexF64, Int64} with 18 stored entries:
 ⎡⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⎤
 ⎢⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⎥
@@ -42,7 +42,7 @@ coef1 (generic function with 1 method)
 
 julia> op = QobjEvo(a, coef1)
 
-Quantum Object Evo.:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=true   isconstant=false
+Quantum Object Evo.:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=true   isconstant=false
 ScalarOperator(0.0 + 0.0im) * MatrixOperator(20 × 20)
 ```
 
@@ -51,7 +51,7 @@ If there are more than 2 operators, we need to put each set of operator and coef
 ```jldoctest qobjevo
 julia> σm = tensor(qeye(10), sigmam())
 
-Quantum Object:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=false
+Quantum Object:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=false
 20×20 SparseMatrixCSC{ComplexF64, Int64} with 10 stored entries:
 ⎡⠂⡀⠀⠀⠀⠀⠀⠀⠀⠀⎤
 ⎢⠀⠀⠂⡀⠀⠀⠀⠀⠀⠀⎥
@@ -64,7 +64,7 @@ coef2 (generic function with 1 method)
 
 julia> op1 = QobjEvo(((a, coef1), (σm, coef2)))
 
-Quantum Object Evo.:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=true   isconstant=false
+Quantum Object Evo.:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=true   isconstant=false
 (ScalarOperator(0.0 + 0.0im) * MatrixOperator(20 × 20) + ScalarOperator(0.0 + 0.0im) * MatrixOperator(20 × 20))
 ```
 
@@ -72,7 +72,7 @@ We can also concretize the operator at a specific time `t`
 ```jldoctest qobjevo
 julia> op1(0.1)
 
-Quantum Object:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=false
+Quantum Object:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=false
 20×20 SparseMatrixCSC{ComplexF64, Int64} with 28 stored entries:
 ⎡⠂⡑⢄⠀⠀⠀⠀⠀⠀⠀⎤
 ⎢⠀⠀⠂⡑⢄⠀⠀⠀⠀⠀⎥
@@ -91,7 +91,7 @@ coef2 (generic function with 1 method)
 
 julia> op1 = QobjEvo(((a, coef1), (σm, coef2)))
 
-Quantum Object Evo.:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=true   isconstant=false
+Quantum Object Evo.:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=true   isconstant=false
 (ScalarOperator(0.0 + 0.0im) * MatrixOperator(20 × 20) + ScalarOperator(0.0 + 0.0im) * MatrixOperator(20 × 20))
 
 julia> p = (ω1 = 1.0, ω2 = 0.5)
@@ -99,7 +99,7 @@ julia> p = (ω1 = 1.0, ω2 = 0.5)
 
 julia> op1(p, 0.1)
 
-Quantum Object:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=false
+Quantum Object:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=false
 20×20 SparseMatrixCSC{ComplexF64, Int64} with 28 stored entries:
 ⎡⠂⡑⢄⠀⠀⠀⠀⠀⠀⠀⎤
 ⎢⠀⠀⠂⡑⢄⠀⠀⠀⠀⠀⎥
@@ -122,10 +122,9 @@ struct QuantumObjectEvolution{
         (type isa Operator || type isa SuperOperator) ||
             throw(ArgumentError("The type $type is not supported for QuantumObjectEvolution."))
 
-        dimensions = _gen_dimensions(dims)
+        dimensions = _gen_dimensions(type, dims)
 
-        _size = _get_size(data)
-        _check_QuantumObject(type, dimensions, _size[1], _size[2])
+        _check_QuantumObject(type, dimensions, size(data, 1), size(data, 2))
 
         return new{ObjType, typeof(dimensions), DT}(data, type, dimensions)
     end
@@ -158,16 +157,13 @@ Generate a [`QuantumObjectEvolution`](@ref) object from a [`SciMLOperator`](http
 Note that `QobjEvo` is a synonym of `QuantumObjectEvolution`
 """
 function QuantumObjectEvolution(data::AbstractSciMLOperator; type = Operator(), dims = nothing)
-    _size = _get_size(data)
     _check_type(type)
 
     if dims isa Nothing
         if type isa Operator
-            dims =
-                (_size[1] == _size[2]) ? Dimensions(_size[1]) :
-                GeneralDimensions(SVector{2}(SVector{1}(_size[1]), SVector{1}(_size[2])))
+            dims = ((size(data, 1),), (size(data, 2),))
         elseif type isa SuperOperator
-            dims = Dimensions(isqrt(_size[2]))
+            dims = ((isqrt(size(data, 1)),), (isqrt(size(data, 2)),))
         end
     end
 
@@ -196,7 +192,7 @@ This operator can be initialized in the same way as the QuTiP `QobjEvo` object.
 ```jldoctest qobjevo
 julia> a = tensor(destroy(10), qeye(2))
 
-Quantum Object:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=false
+Quantum Object:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=false
 20×20 SparseMatrixCSC{ComplexF64, Int64} with 18 stored entries:
 ⎡⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⎤
 ⎢⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⎥
@@ -206,7 +202,7 @@ Quantum Object:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=f
 
 julia> σm = tensor(qeye(10), sigmam())
 
-Quantum Object:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=false
+Quantum Object:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=false
 20×20 SparseMatrixCSC{ComplexF64, Int64} with 10 stored entries:
 ⎡⠂⡀⠀⠀⠀⠀⠀⠀⠀⠀⎤
 ⎢⠀⠀⠂⡀⠀⠀⠀⠀⠀⠀⎥
@@ -222,7 +218,7 @@ coef2 (generic function with 1 method)
 
 julia> op1 = QobjEvo(((a, coef1), (σm, coef2)))
 
-Quantum Object Evo.:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=true   isconstant=false
+Quantum Object Evo.:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=true   isconstant=false
 (ScalarOperator(0.0 + 0.0im) * MatrixOperator(20 × 20) + ScalarOperator(0.0 + 0.0im) * MatrixOperator(20 × 20))
 ```
 
@@ -230,7 +226,7 @@ We can also concretize the operator at a specific time `t`
 ```jldoctest qobjevo
 julia> op1(0.1)
 
-Quantum Object:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=false
+Quantum Object:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=false
 20×20 SparseMatrixCSC{ComplexF64, Int64} with 28 stored entries:
 ⎡⠂⡑⢄⠀⠀⠀⠀⠀⠀⠀⎤
 ⎢⠀⠀⠂⡑⢄⠀⠀⠀⠀⠀⎥
@@ -249,7 +245,7 @@ coef2 (generic function with 1 method)
 
 julia> op1 = QobjEvo(((a, coef1), (σm, coef2)))
 
-Quantum Object Evo.:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=true   isconstant=false
+Quantum Object Evo.:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=true   isconstant=false
 (ScalarOperator(0.0 + 0.0im) * MatrixOperator(20 × 20) + ScalarOperator(0.0 + 0.0im) * MatrixOperator(20 × 20))
 
 julia> p = (ω1 = 1.0, ω2 = 0.5)
@@ -257,7 +253,7 @@ julia> p = (ω1 = 1.0, ω2 = 0.5)
 
 julia> op1(p, 0.1)
 
-Quantum Object:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=false
+Quantum Object:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=false
 20×20 SparseMatrixCSC{ComplexF64, Int64} with 28 stored entries:
 ⎡⠂⡑⢄⠀⠀⠀⠀⠀⠀⠀⎤
 ⎢⠀⠀⠂⡑⢄⠀⠀⠀⠀⠀⎥
@@ -300,7 +296,7 @@ Generate [`QuantumObjectEvolution`](@ref).
 ```jldoctest
 julia> a = tensor(destroy(10), qeye(2))
 
-Quantum Object:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=false
+Quantum Object:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=false
 20×20 SparseMatrixCSC{ComplexF64, Int64} with 18 stored entries:
 ⎡⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⎤
 ⎢⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⎥
@@ -313,7 +309,7 @@ coef (generic function with 1 method)
 
 julia> op = QobjEvo(a, coef)
 
-Quantum Object Evo.:   type=Operator()   dims=[10, 2]   size=(20, 20)   ishermitian=true   isconstant=false
+Quantum Object Evo.:   type=Operator()   dims=([10, 2], [10, 2])   size=(20, 20)   ishermitian=true   isconstant=false
 ScalarOperator(0.0 + 0.0im) * MatrixOperator(20 × 20)
 ```
 """
@@ -448,7 +444,7 @@ Apply the time-dependent [`QuantumObjectEvolution`](@ref) object `A` to the inpu
 ```jldoctest
 julia> a = destroy(20)
 
-Quantum Object:   type=Operator()   dims=[20]   size=(20, 20)   ishermitian=false
+Quantum Object:   type=Operator()   dims=([20], [20])   size=(20, 20)   ishermitian=false
 20×20 SparseMatrixCSC{ComplexF64, Int64} with 19 stored entries:
 ⎡⠈⠢⡀⠀⠀⠀⠀⠀⠀⠀⎤
 ⎢⠀⠀⠈⠢⡀⠀⠀⠀⠀⠀⎥
@@ -464,7 +460,7 @@ coef2 (generic function with 1 method)
 
 julia> A = QobjEvo(((a, coef1), (a', coef2)))
 
-Quantum Object Evo.:   type=Operator()   dims=[20]   size=(20, 20)   ishermitian=true   isconstant=false
+Quantum Object Evo.:   type=Operator()   dims=([20], [20])   size=(20, 20)   ishermitian=true   isconstant=false
 (ScalarOperator(0.0 + 0.0im) * MatrixOperator(20 × 20) + ScalarOperator(0.0 + 0.0im) * MatrixOperator(20 × 20))
 
 julia> ψ1 = fock(20, 3);
@@ -481,7 +477,8 @@ function (A::QuantumObjectEvolution)(
         p,
         t,
     ) where {QobjType <: Union{Ket, OperatorKet}}
-    check_dimensions(A, ψout, ψin)
+    check_mul_dimensions(A, ψin)
+    check_dimensions(ψout, ψin)
 
     if isoper(A) && isoperket(ψin)
         throw(ArgumentError("The input state must be a Ket if the QuantumObjectEvolution object is an Operator."))
