@@ -96,7 +96,8 @@ function ssesolveProblem(
 
     H_eff_evo = _mcsolve_make_Heff_QobjEvo(H, sc_ops_list)
     isoper(H_eff_evo) || throw(ArgumentError("The Hamiltonian must be an Operator."))
-    check_dimensions(H_eff_evo, ψ0)
+
+    check_mul_dimensions(H_eff_evo, ψ0)
     dims = H_eff_evo.dimensions
 
     T = _complex_float_type(Base.promote_eltype(H_eff_evo, ψ0))
@@ -108,18 +109,19 @@ function ssesolveProblem(
 
     # Here the coefficients depend on the state, so this is a non-linear operator, which should be implemented with FunctionOperator instead. However, the nonlinearity is only on the coefficients, and it should be safe.
     K_l = sum(
-        op -> _ScalarOperator_e(op, +) * op + _ScalarOperator_e2_2(op, -) * IdentityOperator(prod(dims)),
+        op ->
+        _ScalarOperator_e(op, +) * op + _ScalarOperator_e2_2(op, -) * IdentityOperator(get_hilbert_size(dims)[1]),
         sc_ops_evo_data,
     )
 
     K = cache_operator(get_data(-1im * QuantumObjectEvolution(H_eff_evo)) + K_l, ψ0)
 
-    D_l = map(op -> op + _ScalarOperator_e(op, -) * IdentityOperator(prod(dims)), sc_ops_evo_data)
+    D_l = map(op -> op + _ScalarOperator_e(op, -) * IdentityOperator(get_hilbert_size(dims)[1]), sc_ops_evo_data)
     D = DiffusionOperator(D_l)
 
     tlist = _check_tlist(tlist, _float_type(T))
 
-    kwargs2 = _merge_saveat(tlist, e_ops, DEFAULT_SDE_SOLVER_OPTIONS; kwargs...)
+    kwargs2 = _merge_saveat(tlist, e_ops, default_sde_solver_options(T); kwargs...)
     kwargs3 = _merge_tstops(kwargs2, isconstant(K), tlist)
     kwargs4 = _generate_stochastic_kwargs(
         e_ops,
@@ -415,7 +417,7 @@ function ssesolve(
     _m_expvals_sol_1 = _get_m_expvals(_sol_1, SaveFuncSSESolve)
 
     normalize_states = Val(false)
-    dims = ens_prob.dimensions
+    dims = ens_prob.dimensions.to
     _expvals_all =
         _expvals_sol_1 isa Nothing ? nothing : map(i -> _get_expvals(sol[:, i], SaveFuncSSESolve), eachindex(sol))
     expvals_all = _expvals_all isa Nothing ? nothing : stack(_expvals_all, dims = 2) # Stack on dimension 2 to align with QuTiP
