@@ -152,7 +152,8 @@ function dfd_mesolveProblem(
     length(ψ0.dimensions) != length(maxdims) &&
         throw(DimensionMismatch("`dim_list` and `maxdims` do not have the same dimension."))
 
-    dim_list = MVector(ψ0.dims)
+    # Use the "to" dimensions for the dim_list (Hilbert space dimensions)
+    dim_list = MVector(dimensions_to_dims(ψ0.dimensions.to))
     H₀ = H(dim_list, dfd_params)
     c_ops₀ = c_ops(dim_list, dfd_params)
     e_ops₀ = e_ops(dim_list, dfd_params)
@@ -363,7 +364,7 @@ function dsf_mesolveProblem(
     αt_list = convert(Vector{T}, α0_l)
     op_l_vec = map(op -> mat2vec(get_data(op)'), op_list)
 
-    dsf_identity = Eye(prod(H₀.dimensions))
+    dsf_identity = Eye(get_hilbert_size(H₀.dimensions)[1])
 
     # Create the Krylov subspace just for initialize
     expv_cache = arnoldi(kron(dsf_identity, dsf_identity), mat2vec(ket2dm(ψ0).data), krylov_dim)
