@@ -14,30 +14,38 @@
         @test_throws ArgumentError Qobj(a, type = OperatorKet())
     end
 
-    # DomainError: incompatible between size of array and type
-    @testset "DomainError" begin
+    # DimensionMismatch: incompatible between size of array and type
+    @testset "DimensionMismatch" begin
         a = rand(ComplexF64, 3, 2)
-        for t in [SuperOperator(), Bra(), OperatorBra()]
-            @test_throws DomainError Qobj(a, type = t)
-        end
+        # SuperOperator requires sqrt-able dimensions (e.g., 4x4 for 2-dim system)
+        @test_throws DimensionMismatch Qobj(a, type = SuperOperator())
+        # Bra requires row vector (1xN), DimensionMismatch when dimensions don't match array size
+        @test_throws DimensionMismatch Qobj(a, type = Bra())
+        # OperatorBra requires row vector, DimensionMismatch when dimensions don't match array size
+        @test_throws DimensionMismatch Qobj(a, type = OperatorBra())
 
         a = rand(ComplexF64, 2, 2, 2)
         for t in (nothing, Ket(), Bra(), Operator(), SuperOperator(), OperatorBra(), OperatorKet())
-            @test_throws DomainError Qobj(a, type = t)
+            @test_throws DimensionMismatch Qobj(a, type = t)
         end
 
+        # Note: (1,2) and (2,1) matrices are now allowed as non-square Operators
+        # Only check SuperOperator which requires sqrt-able dimensions
         a = rand(ComplexF64, 1, 2)
-        @test_throws DomainError Qobj(a, type = Operator())
-        @test_throws DomainError Qobj(a, type = SuperOperator())
+        @test_throws DimensionMismatch Qobj(a, type = SuperOperator())
 
-        @test_throws DomainError Qobj(rand(ComplexF64, 2, 1), type = Operator()) # should be type = Bra
+        # (1,2) becomes a valid Operator with to=(1,), from=(2,)
+        @test Qobj(a, type = Operator()).dimensions == ProductDimensions((HilbertSpace(1),), (HilbertSpace(2),))
 
-        # check that Ket, Bra, SuperOperator, OperatorKet, and OperatorBra don't support GeneralDimensions
-        @test_throws DomainError Qobj(rand(ComplexF64, 2), type = Ket(), dims = ((2,), (1,)))
-        @test_throws DomainError Qobj(rand(ComplexF64, 1, 2), type = Bra(), dims = ((1,), (2,)))
-        @test_throws DomainError Qobj(rand(ComplexF64, 4, 4), type = SuperOperator(), dims = ((2,), (2,)))
-        @test_throws DomainError Qobj(rand(ComplexF64, 4), type = OperatorKet(), dims = ((2,), (1,)))
-        @test_throws DomainError Qobj(rand(ComplexF64, 1, 4), type = OperatorBra(), dims = ((1,), (2,)))
+        # (2,1) becomes a valid Operator with to=(2,), from=(1,)
+        @test Qobj(rand(ComplexF64, 2, 1), type = Operator()).dimensions == ProductDimensions((HilbertSpace(2),), (HilbertSpace(1),))
+
+        # check non-square dimensions work for all types
+        @test Qobj(rand(ComplexF64, 2), type = Ket(), dims = ((2,), (1,))).dimensions.to == (HilbertSpace(2),)
+        @test Qobj(rand(ComplexF64, 1, 2), type = Bra(), dims = ((1,), (2,))).dimensions.from == (HilbertSpace(2),)
+        @test Qobj(rand(ComplexF64, 4, 9), type = SuperOperator(), dims = ((2,), (3,))).dimensions.to == (HilbertSpace(2),)
+        @test Qobj(rand(ComplexF64, 4), type = OperatorKet(), dims = ((2,), (1,))).dimensions.to == (HilbertSpace(2),)
+        @test Qobj(rand(ComplexF64, 1, 4), type = OperatorBra(), dims = ((1,), (2,))).dimensions.from == (HilbertSpace(2),)
     end
 
     # unsupported type of dims
@@ -89,7 +97,7 @@
         a = sprand(ComplexF64, 100, 100, 0.1)
         a2 = Qobj(a)
         a3 = Qobj(a, type = SuperOperator())
-        a4 = Qobj(sprand(ComplexF64, 100, 10, 0.1)) # GeneralDimensions
+        a4 = Qobj(sprand(ComplexF64, 100, 10, 0.1)) # non-square ProductDimensions
         a5 = QuantumObject(rand(ComplexF64, 2 * 3 * 4, 5), dims = ((2, 3, 4), (5,)))
         @test isket(a2) == false
         @test isbra(a2) == false
@@ -100,7 +108,7 @@
         @test iscached(a2) == true
         @test isconstant(a2) == true
         @test isunitary(a2) == false
-        @test a2.dims == [100]
+        @test a2.dims == ([100], [100])
         @test isket(a3) == false
         @test isbra(a3) == false
         @test isoper(a3) == false
@@ -110,7 +118,7 @@
         @test iscached(a3) == true
         @test isconstant(a3) == true
         @test isunitary(a3) == false
-        @test a3.dims == [10]
+        @test a3.dims == ([10], [10])
         @test isket(a4) == false
         @test isbra(a4) == false
         @test isoper(a4) == true
@@ -120,9 +128,9 @@
         @test iscached(a4) == true
         @test isconstant(a4) == true
         @test isunitary(a4) == false
-        @test a4.dims == [[100], [10]]
+        @test a4.dims == ([100], [10])
         @test isoper(a5) == true
-        @test a5.dims == [[2, 3, 4], [5]]
+        @test a5.dims == ([2, 3, 4], [5])
         @test_throws DimensionMismatch Qobj(a, dims = 2)
         @test_throws DimensionMismatch Qobj(a4.data, dims = 2)
         @test_throws DimensionMismatch Qobj(a4.data, dims = ((100,), (2,)))
@@ -150,12 +158,12 @@
         @test isoperket(ρ_bra) == false
         @test isoperbra(ρ_bra) == true
         @test isunitary(ρ_bra) == false
-        @test ρ_bra.dims == [2]
-        @test ρ_ket.dims == [2]
+        @test ρ_bra.dims == ([1], [2])
+        @test ρ_ket.dims == ([2], [1])
         @test H * ρ ≈ spre(H) * ρ
         @test ρ * H ≈ spost(H) * ρ
         @test H * ρ * H ≈ sprepost(H, H) * ρ
-        @test (L * ρ_ket).dims == [2]
+        @test (L * ρ_ket).dims == ([2], [1])
         @test L * ρ_ket ≈ -1im * (+(spre(H) * ρ_ket) - spost(H) * ρ_ket)
         @test (ρ_bra * L')' == L * ρ_ket
         @test sum((conj(ρ) .* ρ).data) ≈ dot(ρ_ket, ρ_ket) ≈ ρ_bra * ρ_ket
@@ -242,6 +250,16 @@
         @test X == a
         tril!(X)
         @test nnz(X) == 0
+
+        # Test a more complex dimension case
+        O = rand_dm(3) ⊗ rand_dm(3) ⊗ rand_dm(3)
+        ψ1 = rand_ket(3) ⊗ rand_ket(3) ⊗ rand_ket(3)
+        ψ2 = rand_ket(3) ⊗ rand_ket(3) ⊗ rand_ket(3)
+        Π = QuantumObject(hcat(ψ1.data, ψ2.data), dims = ((3, 3, 3), (2,)))
+
+        o = (Π' * O * Π)
+        @test o.dimensions == ProductDimensions((HilbertSpace(2),), (HilbertSpace(2),))
+        @test isoper(o) == true
     end
 
     @testset "broadcasting" begin
@@ -271,11 +289,11 @@
         @test opstring ==
             "\nQuantum Object:   type=Operator()   dims=$a_dims   size=$a_size   ishermitian=$a_isherm\n$datastring"
 
-        # GeneralDimensions
+        # non-square ProductDimensions
         Gop = tensor(a, ψ)
         opstring = sprint((t, s) -> show(t, "text/plain", s), Gop)
         datastring = sprint((t, s) -> show(t, "text/plain", s), Gop.data)
-        Gop_dims = [[N, N], [N, 1]]
+        Gop_dims = ([N, N], [N, 1])  # Tuple of vectors for non-square operator
         Gop_size = size(Gop)
         Gop_isherm = isherm(Gop)
         @test opstring ==
@@ -367,8 +385,8 @@
             end
 
             UnionType = Union{
-                QuantumObject{Bra, Dimensions{1, Tuple{Space}}, Matrix{T}},
-                QuantumObject{Operator, Dimensions{1, Tuple{Space}}, Matrix{T}},
+                QuantumObject{Bra, ProductDimensions{1, 1, Tuple{HilbertSpace}, Tuple{HilbertSpace}}, Matrix{T}},
+                QuantumObject{Operator, ProductDimensions{1, 1, Tuple{HilbertSpace}, Tuple{HilbertSpace}}, Matrix{T}},
             }
             a = rand(T, 1, N)
             @inferred UnionType Qobj(a)
@@ -376,13 +394,9 @@
                 @inferred Qobj(a, type = type)
             end
 
-            UnionType2 = Union{
-                QuantumObject{Operator, GeneralDimensions{1, 1, Tuple{Space}, Tuple{Space}}, Matrix{T}},
-                QuantumObject{Operator, Dimensions{1, Tuple{Space}}, Matrix{T}},
-            }
             a = rand(T, N, N)
             @inferred UnionType Qobj(a)
-            @inferred UnionType2 Qobj(a, type = Operator())
+            @inferred Qobj(a, type = Operator())
             @inferred Qobj(a, type = SuperOperator())
         end
 
@@ -670,7 +684,7 @@
         ρ1_ptr = ptrace(ρ, 1)
         ρ2_ptr = ptrace(ρ, 2)
 
-        # use GeneralDimensions to do partial trace
+        # use non-square ProductDimensions to do partial trace
         ρ1_compound = Qobj(zeros(ComplexF64, 2, 2), dims = ((2, 1), (2, 1)))
         II = qeye(2)
         basis_list = [basis(2, i) for i in 0:1]
@@ -741,7 +755,7 @@
         @test_throws ArgumentError ptrace(ρtotal, (0, 2))
         @test_throws ArgumentError ptrace(ρtotal, (2, 5))
         @test_throws ArgumentError ptrace(ρtotal, (2, 2, 3))
-        @test_throws ArgumentError ptrace(Qobj(zeros(ComplexF64, 3, 2)), 1) # invalid GeneralDimensions
+        @test_throws ArgumentError ptrace(Qobj(zeros(ComplexF64, 3, 2)), 1) # invalid non-square ProductDimensions
 
         @testset "Type Inference (ptrace)" begin
             @inferred ptrace(ρ, 1)
@@ -754,7 +768,7 @@
     end
 
     @testset "permute" begin
-        # standard Dimensions
+        # standard ProductDimensions
         ket_a = Qobj(rand(ComplexF64, 2))
         ket_b = Qobj(rand(ComplexF64, 3))
         ket_c = Qobj(rand(ComplexF64, 4))
@@ -776,9 +790,9 @@
         @test ket_bdca ≈ tensor(ket_b, ket_d, ket_c, ket_a)
         @test bra_bdca ≈ tensor(bra_b, bra_d, bra_c, bra_a)
         @test op_bdca ≈ tensor(op_b, op_d, op_c, op_a)
-        @test ket_bdca.dims == correct_dims
-        @test bra_bdca.dims == correct_dims
-        @test op_bdca.dims == correct_dims
+        @test ket_bdca.dims == (correct_dims, [1, 1, 1, 1])
+        @test bra_bdca.dims == ([1, 1, 1, 1], correct_dims)
+        @test op_bdca.dims == (correct_dims, correct_dims)
         @test isket(ket_bdca)
         @test isbra(bra_bdca)
         @test isoper(op_bdca)
@@ -789,14 +803,14 @@
         @test_throws ArgumentError permute(op_bdca, wrong_order1)
         @test_throws ArgumentError permute(op_bdca, wrong_order2)
 
-        # GeneralDimensions
+        # non-square ProductDimensions
         Gop_d = Qobj(rand(ComplexF64, 5, 6))
         compound_bdca = permute(tensor(ket_a, op_b, bra_c, Gop_d), (2, 4, 3, 1))
         compound_dacb = permute(tensor(ket_a, op_b, bra_c, Gop_d), (4, 1, 3, 2))
         @test compound_bdca ≈ tensor(op_b, Gop_d, bra_c, ket_a)
         @test compound_dacb ≈ tensor(Gop_d, ket_a, bra_c, op_b)
-        @test compound_bdca.dims == [[3, 5, 1, 2], [3, 6, 4, 1]]
-        @test compound_dacb.dims == [[5, 2, 1, 3], [6, 1, 4, 3]]
+        @test compound_bdca.dims == ([3, 5, 1, 2], [3, 6, 4, 1])
+        @test compound_dacb.dims == ([5, 2, 1, 3], [6, 1, 4, 3])
         @test isoper(compound_bdca)
         @test isoper(compound_dacb)
 
